@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2017	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -86,12 +86,15 @@ void pool_query_context_destroy(POOL_QUERY_CONTEXT *query_context)
 	if (query_context)
 	{
 		MemoryContext memory_context = query_context->memory_context;
-
-		ereport(DEBUG1,
-				(errmsg("pool_query_context_destroy: query context:%p", query_context)));
-
 		session_context = pool_get_session_context(false);
 		pool_unset_query_in_progress();
+		if (query_context->pg_terminate_backend_conn)
+		{
+			ereport(DEBUG1,
+				 (errmsg("resetting the connection flag for pg_terminate_backend")));
+			pool_unset_connection_will_be_terminated(query_context->pg_terminate_backend_conn);
+		}
+		query_context->pg_terminate_backend_conn = NULL;
 		query_context->original_query = NULL;
 		session_context->query_context = NULL;
 		pfree(query_context);
@@ -99,21 +102,6 @@ void pool_query_context_destroy(POOL_QUERY_CONTEXT *query_context)
 	}
 }
 
-/*
- * Perform shallow copy of given query context. Used in parse_before_bind.
- */
-POOL_QUERY_CONTEXT *pool_query_context_shallow_copy(POOL_QUERY_CONTEXT *query_context)
-{
-	POOL_QUERY_CONTEXT *qc;
-	MemoryContext memory_context;
-
-	qc = pool_init_query_context();
-	memory_context = qc->memory_context;
-	memcpy(qc, query_context, sizeof(POOL_QUERY_CONTEXT));
-	qc->memory_context = memory_context;
-	return qc;
-}
-
 /*
  * Start query
  */
@@ -137,7 +125,6 @@ void pool_start_query(POOL_QUERY_CONTEXT *query_context, char *query, int len, N
 		if (pool_config->memory_cache_enabled)
 			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
-		query_context->skip_cache_commit = false;
 		session_context->query_context = query_context;
 		MemoryContextSwitchTo(old_context);
 	}
@@ -211,7 +198,7 @@ void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 			 * send query.
 			 */
 			if (pool_config->master_slave_mode &&
-				!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+				pool_config->master_slave_sub_mode == STREAM_MODE &&
 				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
 			{
 				continue;
@@ -292,29 +279,13 @@ int pool_virtual_master_db_node_id(void)
 {
 	POOL_SESSION_CONTEXT *sc;
 
-	/*
-	 * Check whether failover is in progress. If so, just abort this session.
-	 */
-	if (Req_info->switching)
-	{
-#ifdef NOT_USED
-		POOL_SETMASK(&BlockSig);
-		ereport(WARNING,
-				(errmsg("failover/failback is in progress"),
-						errdetail("executing failover or failback on backend"),
-				 errhint("In a moment you should be able to reconnect to the database")));
-		POOL_SETMASK(&UnBlockSig);
-#endif
-		child_exit(POOL_EXIT_AND_RESTART);
-	}
-
 	sc = pool_get_session_context(true);
 	if (!sc)
 	{
 		return REAL_MASTER_NODE_ID;
 	}
 
-	if (sc->in_progress && sc->query_context)
+	if (sc->query_context)
 	{
 		int node_id = sc->query_context->virtual_master_node_id;
 
@@ -324,16 +295,24 @@ int pool_virtual_master_db_node_id(void)
 			  * Make sure that virtual_master_node_id is either primary node
 			  * id or load balance node id.  If not, it is likely that
 			  * virtual_master_node_id is not set up yet. Let's use the
-			  * primary node id.
+			  * primary node id. except for the special case where we need
+			  * to send the query to the node which is not primary nor the
+			  * load balance node. Currently there is only one special such
+			  * case that is handling of pg_terminate_backend() function, which
+			  * may refer to the backend connection that is neither hosted by
+			  * the primary or load balance node for current child process, but
+			  * the query must be forwarded to that node. Since only that backend
+			  * node can handle that pg_terminate_backend query
+			  *
 			  */
-
-			ereport(DEBUG1,
-					(errmsg("pool_virtual_master_db_node_id: virtual_master_node_id:%d load_balance_node_id:%d PRIMARY_NODE_ID:%d",
-							node_id, sc->load_balance_node_id, PRIMARY_NODE_ID)));
-
 			if (node_id != sc->load_balance_node_id && node_id != PRIMARY_NODE_ID)
 			{
-				node_id = PRIMARY_NODE_ID;
+				/*
+				 * Only return the primary node id if we are not processing the
+				 * pg_terminate_backend query
+				 */
+				if (sc->query_context->pg_terminate_backend_conn == NULL)
+					node_id = PRIMARY_NODE_ID;
 			}
 		}
 
@@ -347,17 +326,33 @@ int pool_virtual_master_db_node_id(void)
 	 */
 	if (MASTER_SLAVE)
 	{
-		int node_id;
-
-		node_id = pool_get_preferred_master_node_id();
-		if (node_id >= 0)
-			return node_id;
-
 		return PRIMARY_NODE_ID;
 	}
 	return my_master_node_id;
 }
 
+/*
+ * The function sets the destination for the current query to the specific backend node
+ */
+void pool_force_query_node_to_backend(POOL_QUERY_CONTEXT *query_context, int backend_id)
+{
+	int i;
+	CHECK_QUERY_CONTEXT_IS_VALID;
+
+	ereport(DEBUG1,
+		(errmsg("forcing query destination node to backend node:%d",backend_id)));
+
+	pool_set_node_to_be_sent(query_context,backend_id);
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		if (query_context->where_to_send[i])
+		{
+			query_context->virtual_master_node_id = i;
+			break;
+		}
+	}
+}
+
 /*
  * Decide where to send queries(thus expecting response)
  */
@@ -418,7 +413,10 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 		 * statement queries, but until that day we need this band
 		 * aid.
 		 */
-		pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+		if (query_context->is_multi_statement)
+		{
+			pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+		}
 	}
 	else if (MASTER_SLAVE)
 	{
@@ -427,7 +425,7 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 		dest = send_to_where(node, query);
 
 		ereport(DEBUG1,
-			(errmsg("decide where to send the query"),
+			(errmsg("decide where to send the queries"),
 				 errdetail("destination = %d for query= \"%s\"", dest, query)));
 
 		/* Should be sent to primary only? */
@@ -456,15 +454,6 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 				 *	transaction isolation level is not SERIALIZABLE)
 				 * we might be able to load balance.
 				 */
-
-				ereport(DEBUG1,
-						(errmsg("checking load balance precondtions. TSTATE:%c wrting_trancation:%d failed_transaction:%d isolation:%d",
-								TSTATE(backend, PRIMARY_NODE_ID),
-								pool_is_writing_transaction(),
-								pool_is_failed_transaction(),
-								pool_get_transaction_isolation()),
-						 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 				if (TSTATE(backend, PRIMARY_NODE_ID) == 'I' ||
 					(!pool_is_writing_transaction() &&
 					 !pool_is_failed_transaction() &&
@@ -479,14 +468,10 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					/*
 					 * If replication delay is too much, we prefer to send to the primary.
 					 */
-					if (!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+					if (pool_config->master_slave_sub_mode == STREAM_MODE &&
 						pool_config->delay_threshold &&
 						bkinfo->standby_delay > pool_config->delay_threshold)
 					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because of too much replication delay"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
@@ -496,10 +481,6 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					 */
 					else if (pool_has_function_call(node))
 					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because writing functions are used"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
@@ -517,10 +498,6 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					 */
 					else if (pool_has_system_catalog(node))
 					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because systems catalogs are used"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
@@ -530,10 +507,6 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					 */
 					else if (pool_config->check_temp_table && pool_has_temp_table(node))
 					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because temporary tables are used"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
@@ -543,10 +516,6 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					 */
 					else if (pool_config->check_unlogged_table && pool_has_unlogged_table(node))
 					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because unlogged tables are used"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
@@ -633,9 +602,9 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 	{
 		POOL_SENT_MESSAGE *msg;
 
-		msg = pool_get_sent_message('Q', ((ExecuteStmt *)node)->name, POOL_SENT_MESSAGE_CREATED);
+		msg = pool_get_sent_message('Q', ((ExecuteStmt *)node)->name);
 		if (!msg)
-			msg = pool_get_sent_message('P', ((ExecuteStmt *)node)->name, POOL_SENT_MESSAGE_CREATED);
+			msg = pool_get_sent_message('P', ((ExecuteStmt *)node)->name);
 		if (msg)
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
@@ -938,6 +907,11 @@ POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 			per_node_statement_log(backend, i, msgbuf);
 		}
 
+		/* Set sync map so that we could wait for response from appropreate node */
+		pool_set_sync_map(i);
+		ereport(DEBUG1,
+				(errmsg("pool_send_and_wait: pool_set_sync_map: %d", i)));
+
 		/* if Execute message, count up stats count */
 		if (*kind == 'E')
 		{
@@ -946,11 +920,11 @@ POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 
 		send_extended_protocol_message(backend, i, kind, str_len, str);
 
-		if ((*kind == 'P' || *kind == 'E' || *kind == 'C') && STREAM)
+		if (*kind == 'E' && STREAM)
 		{
 			/*
 			 * Send flush message to backend to make sure that we get any response
-			 * from backend in Streaming replication mode.
+			 * from backend in Sream replication mode.
 			 */
 
 			POOL_CONNECTION *cp = CONNECTION(backend, i);
@@ -959,9 +933,6 @@ POOL_STATUS pool_extended_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 			pool_write(cp, "H", 1);
 			len = htonl(sizeof(len));
 			pool_write_and_flush(cp, &len, sizeof(len));
-
-			ereport(DEBUG1,
-					(errmsg("pool_send_and_wait: send flush message to %d", i)));
 		}
 	}
 
@@ -1181,11 +1152,19 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		}
 
 		/*
-		 * COPY FROM
+		 * COPY
 		 */
 		else if (IsA(node, CopyStmt))
 		{
-			return (((CopyStmt *)node)->is_from)?POOL_PRIMARY:POOL_EITHER;
+			if (((CopyStmt *)node)->is_from)
+				return POOL_PRIMARY;
+			else
+			{
+				if (((CopyStmt *)node)->query == NULL)
+					return POOL_EITHER;
+				else
+					return (IsA(((CopyStmt *)node)->query, SelectStmt))?POOL_EITHER:POOL_PRIMARY;
+			}
 		}
 
 		/*
@@ -1351,13 +1330,6 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			 */
 			return POOL_PRIMARY; 
 		}
-		/*
-		 * SHOW
-		 */
-		else if (IsA(node, VariableShowStmt))
-		{
-			return POOL_EITHER;
-		}
 
 		/*
 		 * Other statements are sent to primary
@@ -1384,9 +1356,9 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 	}
 	else
 	{
-		msg = pool_get_sent_message('Q', d->name, POOL_SENT_MESSAGE_CREATED);
+		msg = pool_get_sent_message('Q', d->name);
 		if (!msg)
-			msg = pool_get_sent_message('P', d->name, POOL_SENT_MESSAGE_CREATED);
+			msg = pool_get_sent_message('P', d->name);
 		if (msg)
 		{
 			/* Inherit same map from PREPARE or PARSE */
