@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -54,12 +54,12 @@
 #include "context/pool_process_context.h"
 #include "context/pool_session_context.h"
 #include "pool_config.h"
+#include "pool_config_variables.h"
 #include "utils/pool_ip.h"
 #include "utils/pool_stream.h"
 #include "utils/elog.h"
 #include "auth/md5.h"
 #include "auth/pool_passwd.h"
-#include "utils/pool_relcache.h"
 
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
@@ -91,17 +91,12 @@ static bool backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION
 static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp);
 static int choose_db_node_id(char *str);
 static void child_will_go_down(int code, Datum arg);
-static int opt_sort(const void *a, const void *b);
 /*
- * Non 0 means SIGTERM (smart shutdown) or SIGINT (fast shutdown) has arrived
+ * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
 volatile sig_atomic_t exit_request = 0;
 static volatile sig_atomic_t alarm_enabled = false;
 
-/*
- * Ignore SIGUSR1 if requested. Used when DROP DATABASE is requested.
- */
-volatile sig_atomic_t ignore_sigusr1 = 0;
 
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
@@ -122,10 +117,6 @@ char remote_host[NI_MAXHOST];	/* client host */
 char remote_port[NI_MAXSERV];	/* client port */
 POOL_CONNECTION* volatile child_frontend = NULL;
 
-#ifdef DEBUG
-bool stop_now = false;
-#endif
-
 /*
 * child main loop
 */
@@ -239,10 +230,7 @@ void do_child(int *fds)
 		MemoryContextSwitchTo(ProcessLoopContext);
 
 		if (accepted)
-		{
-			accepted = 0;
 			connection_count_down();
-		}
         
         backend_cleanup(&child_frontend, backend, frontend_invalid);
 
@@ -291,8 +279,7 @@ void do_child(int *fds)
 		StartupPacket *sp;
 		int front_end_fd;
 		SockAddr saddr;
-
-		/* reset per iteration memory context */
+		/* rest per iteration memory context */
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
 
@@ -500,6 +487,8 @@ backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volat
             pool_discard_cp(sp->user, sp->database, sp->major);
     }
 
+	/* reset the config parameters */
+	reset_all_variables(NULL,NULL);
     return cache_connection;
 }
 
@@ -515,11 +504,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	int protov;
 	int len;
 	char *p;
-	char **guc_options;
-	int opt_num = 0;
-	char *sp_sort;
-	char *tmpopt;
-	int i;
 
 	sp = (StartupPacket *)palloc0(sizeof(*sp));
 	enable_authentication_timeout();
@@ -546,6 +530,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	memcpy(&protov, sp->startup_packet, sizeof(protov));
 	sp->major = ntohl(protov)>>16;
 	sp->minor = ntohl(protov) & 0x0000ffff;
+	p = sp->startup_packet;
     cp->protoVersion = sp->major;
 
 	switch(sp->major)
@@ -562,46 +547,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			break;
 
 		case PROTO_MAJOR_V3: /* V3 */
-			/* copy startup_packet */
-			sp_sort = palloc0(len);
-			memcpy(sp_sort,sp->startup_packet,len);
-
-			p = sp_sort;
-			p += sizeof(int);	/* skip protocol version info */
-			/* count the number of options */
-			while(*p)
-			{
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-				opt_num ++;
-			}
-			guc_options = (char **)palloc0(opt_num * sizeof(char *));
-			/* get guc_option name list */
-			p = sp_sort + sizeof(int);
-			for(i = 0; i < opt_num; i++)
-			{
-				guc_options[i] = p;
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-			}
-			/* sort option name using quick sort */
-			qsort( (void *)guc_options, opt_num, sizeof(char *), opt_sort );
-
-			p = sp->startup_packet + sizeof(int);	/* skip protocol version info */
-			for (i = 0; i < opt_num; i++)
-			{
-				tmpopt = guc_options[i];
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option name */
-				p += (strlen(tmpopt) + 1);
-				tmpopt += (strlen(tmpopt) + 1);
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option value */
-				p += (strlen(tmpopt) + 1);
-			}
-
-			pfree(guc_options);
-			pfree(sp_sort);
-
-			p = sp->startup_packet;
 			p += sizeof(int);	/* skip protocol version info */
 
 			while(*p)
@@ -635,10 +580,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 						(errmsg("reading startup packet"),
 							 errdetail("application_name: %s", p)));
 				}
-				else
-				{
-					p += (strlen(p) + 1);
-				}
 
 				p += (strlen(p) + 1);
 			}
@@ -856,8 +797,6 @@ void cancel_request(CancelPacket *sp)
 		if (con == NULL)
 			return;
 
-		pool_set_db_node_id(con, i);
-
 		len = htonl(sizeof(len) + sizeof(CancelPacket));
 		pool_write(con, &len, sizeof(len));
 
@@ -914,20 +853,19 @@ static StartupPacket *StartupPacketCopy(StartupPacket *sp)
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend)
 {
 	POOL_CONNECTION_POOL *backend;
-	StartupPacket *volatile topmem_sp = NULL;
-	volatile bool	topmem_sp_set = false;
-	int			i;
+	StartupPacket *topmem_sp;
+	int i;
 
 	/* connect to the backend */
 	backend = pool_create_cp();
 	if (backend == NULL)
 	{
-		pool_send_error_message(frontend, sp->major, "XX000", "all backend nodes are down, pgpool requires at least one valid node", "",
-								"repair the backend nodes and restart pgpool", __FILE__, __LINE__);
+		pool_send_error_message(frontend, sp->major, "XX000", "connection cache is full", "",
+								"increase max_pool", __FILE__, __LINE__);
 		ereport(ERROR,
-				(errmsg("unable to connect to backend"),
-				 errdetail("all backend nodes are down, pgpool requires at least one valid node"),
-				 errhint("repair the backend nodes and restart pgpool")));
+			(errmsg("unable to connect to backend"),
+				errdetail("connection cache is full"),
+					errhint("increase the \"max_pool\" configuration value, current max_pool is %d",pool_config->max_pool)));
 	}
 	
 	PG_TRY();
@@ -942,7 +880,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 			{
 
 				/* set DB node id */
-				pool_set_db_node_id(CONNECTION(backend, i), i);
+				CONNECTION(backend, i)->db_node_id = i;
 
 				/* mark this is a backend connection */
 				CONNECTION(backend, i)->isbackend = 1;
@@ -953,7 +891,6 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 				 * save startup packet info
 				 */
 				CONNECTION_SLOT(backend, i)->sp = topmem_sp;
-				topmem_sp_set = true;
 
 				/* send startup packet */
 				send_startup_packet(CONNECTION_SLOT(backend, i));
@@ -967,17 +904,10 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	PG_CATCH();
 	{
 		pool_discard_cp(sp->user, sp->database, sp->major);
-		topmem_sp = NULL;
 		PG_RE_THROW();
 	}
 	PG_END_TRY();
 
-	/* At this point, we need to free previously allocated memory for the
-	 * startup packet if no backend is up.
-	 */
-	if (!topmem_sp_set && topmem_sp != NULL)
-		pfree(topmem_sp);
-
 	return backend;
 }
 
@@ -986,16 +916,10 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
  */
 static RETSIGTYPE die(int sig)
 {
-	int save_errno;
+	int save_errno = errno;
 
-	POOL_SETMASK(&BlockSig);
-
-	save_errno = errno;
-
-#ifdef NOT_USED
 	ereport(LOG,
 			(errmsg("child process received shutdown request signal %d", sig)));
-#endif
 
 	exit_request = sig;
 
@@ -1005,39 +929,32 @@ static RETSIGTYPE die(int sig)
 			/* Refuse further requests by closing listen socket */
 			if (child_inet_fd)
 			{
-#ifdef NOT_USED
 				ereport(LOG,
 						(errmsg("closing listen socket")));
-#endif
 				close(child_inet_fd);
 			}
 			close(child_unix_fd);
 
 			if (idle == 0)
 			{
-#ifdef NOT_USED
 				ereport(DEBUG1,
 						(errmsg("smart shutdown request received, but child is not in idle state")));
-#endif
 			}
 			break;
 
 		case SIGINT:	/* fast shutdown */
 		case SIGQUIT:	/* immediate shutdown */
-			POOL_SETMASK(&UnBlockSig);
 			child_exit(POOL_EXIT_NO_RESTART);
 			break;
 		default:
-#ifdef NOT_USED
 			ereport(LOG,
 				(errmsg("child process received unknown signal: %d",sig),
 					 errdetail("ignoring...")));
-#endif
+
 			break;
 	}
 
 	errno = save_errno;
-	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1051,16 +968,8 @@ static RETSIGTYPE close_idle_connection(int sig)
 	ConnectionInfo *info;
 	int save_errno = errno;
 
-	/*
-	 * DROP DATABSE is ongoing.
-	 */
-	if (ignore_sigusr1)
-		return;
-
-#ifdef NOT_USED
 	ereport(DEBUG1,
 			(errmsg("close connection request received")));
-#endif
 
 	for (j=0;j<pool_config->max_pool;j++, p++)
 	{
@@ -1073,12 +982,9 @@ static RETSIGTYPE close_idle_connection(int sig)
 
 		if (MASTER_CONNECTION(p)->closetime > 0)		/* idle connection? */
 		{
-#ifdef NOT_USED
 			ereport(DEBUG1,
 					(errmsg("closing idle connection"),
 					 errdetail("user: %s database: %s", MASTER_CONNECTION(p)->sp->user, MASTER_CONNECTION(p)->sp->database)));
-#endif
-
 			pool_send_frontend_exits(p);
 
 			for (i=0;i<NUM_BACKENDS;i++)
@@ -1122,7 +1028,8 @@ static void enable_authentication_timeout(void)
 {
 	if(pool_config->authentication_timeout <= 0)
 		return;
-	pool_alarm(authentication_timeout, pool_config->authentication_timeout);
+	pool_signal(SIGALRM, authentication_timeout);
+	alarm(pool_config->authentication_timeout);
 	alarm_enabled = true;
 }
 
@@ -1130,7 +1037,8 @@ static void disable_authentication_timeout(void)
 {
 	if(alarm_enabled)
 	{
-		pool_undo_alarm();
+		alarm(0);
+		pool_signal(SIGALRM, SIG_IGN);
 		alarm_enabled = false;
 	}
 }
@@ -1236,14 +1144,8 @@ void child_exit(int code)
 	if(processType != PT_CHILD)
 	{
 		/* should never happen */
-
-		/* Remove call to ereport because child_exit() is called inside a
-		 * signal handler.
-		 */
-#ifdef NOT_USED
         ereport(WARNING,
                 (errmsg("child_exit: called from invalid process. ignored.")));
-#endif
 		return;
 	}
 	exit(code);
@@ -1253,7 +1155,7 @@ void child_exit(int code)
  * create a persistent connection
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	int db_node_id,	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
+	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
 	POOL_CONNECTION_POOL_SLOT *cp;
 	int fd;
@@ -1297,8 +1199,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	cp->con = pool_open(fd,true);
 	cp->closetime = 0;
 	cp->con->isbackend = 1;
-	pool_set_db_node_id(cp->con, db_node_id);
-
 	pool_ssl_negotiate_clientserver(cp->con);
 
 	/*
@@ -1375,18 +1275,17 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
  * make_persistent_db_connection() which does not ereports in case of an error
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection_noerror(
-	int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry)
+                        char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
     POOL_CONNECTION_POOL_SLOT *slot = NULL;
     MemoryContext oldContext = CurrentMemoryContext;
     PG_TRY();
     {
-        slot = make_persistent_db_connection(db_node_id,
-											 hostname,
-											 port,
-											 dbname,
-											 user,
-											 password, retry);
+        slot = make_persistent_db_connection(hostname,
+                                                 port,
+                                                 dbname,
+                                                 user,
+                                                 password, retry);
     }
     PG_CATCH();
     {
@@ -1474,9 +1373,24 @@ static void s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password)
 
 	if (kind != 'R')
 	{
-        ereport(ERROR,
-			(errmsg("failed to authenticate"),
-				errdetail("invalid authentication message response type, Expecting 'R' and received '%c'",kind)));
+		char *msg;
+		int sts = 0;
+
+		if (kind == 'E' || kind == 'N')
+		{
+			sts =  pool_extract_error_message(false, cp->con, cp->sp->major, false, &msg);
+		}
+
+		if (sts == 1)	/* succeeded in extracting error/notice message */
+		{
+			ereport(ERROR,
+					(errmsg("failed to authenticate"),
+					 errdetail("%s", msg)));
+			pfree(msg);
+		}
+			ereport(ERROR,
+					(errmsg("failed to authenticate"),
+					 errdetail("invalid authentication message response type, Expecting 'R' and received '%c'",kind)));
 	}
 
 	/* read message length */
@@ -1919,7 +1833,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	int fd = 0;
 	int afd;
 	int *walk;
-	int on;
 
 #ifdef ACCEPT_PERFORMANCE
 	struct timeval now1, now2;
@@ -1963,50 +1876,7 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	 */
 	if (SERIALIZE_ACCEPT)
 	{
-		if (pool_config->connection_life_time == 0)
-		{
-			pool_semaphore_lock(ACCEPT_FD_SEM);
-		}
-		else
-		{
-			int sts;
-
-			for (;;)
-			{
-				sts = pool_semaphore_lock_allow_interrupt(ACCEPT_FD_SEM);
-
-				/* Interrupted by alarm */
-				if (sts == -2)
-				{
-					/*
-					 * Check if there are expired connection_life_time.
-					 */
-					if (backend_timer_expired)
-					{
-						/*
-						 * We add 10 seconds to connection_life_time so that there's
-						 * enough margin.
-						 */
-						int	seconds = pool_config->connection_life_time + 10;
-
-						while (seconds-- > 0)
-						{
-							/* check backend timer is expired */
-							if (backend_timer_expired)
-							{
-								pool_backend_timer();
-								backend_timer_expired = 0;
-								break;
-							}
-							sleep(1);
-						}
-					}
-				}
-				else	/* success or other error */
-					break;
-			}
-		}
-
+		pool_semaphore_lock(ACCEPT_FD_SEM);
 		set_ps_display("wait for connection request", false);
 		ereport(DEBUG1,
 			   (errmsg("LOCKING select()")));
@@ -2134,30 +2004,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 		return RETRY;
 
 	}
-
-	/*
-	 * Set no delay if AF_INET socket. Not sure if this is really necessary
-	 * but PostgreSQL does this.
-	 */
-	if (!FD_ISSET(fds[0], &rmask))	/* fds[0] is UNIX domain socket */
-	{
-		on = 1;
-		if (setsockopt(afd, IPPROTO_TCP, TCP_NODELAY,
-					   (char *) &on,
-					   sizeof(on)) < 0)
-		{
-			ereport(WARNING,
-					(errmsg("wait_for_new_connections: setsockopt failed with error \"%s\"",strerror(errno))));
-			close(afd);
-			return -1;
-		}
-	}
-
-	/*
-	 * Make sure that the socket is non blocking.
-	 */
-	pool_unset_nonblock(afd);
-
 #ifdef ACCEPT_PERFORMANCE
 	gettimeofday(&now2,0);
 	atime += (now2.tv_sec - now1.tv_sec)*1000000 + (now2.tv_usec - now1.tv_usec);
@@ -2176,7 +2022,7 @@ static void check_config_reload(void)
 	if (got_sighup)
 	{
         MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
-		pool_get_config(get_config_file_name(), RELOAD_CONFIG);
+		pool_get_config(get_config_file_name(), CFGCXT_RELOAD);
         MemoryContextSwitchTo(oldContext);
 		if (pool_config->enable_pool_hba)
 		{
@@ -2230,7 +2076,7 @@ static void validate_backend_connectivity(int front_end_fd)
 		if(front_end_fd > 0)
 		{
 			POOL_CONNECTION *cp;
-			StartupPacket *volatile sp = NULL;
+			StartupPacket *sp;
 
 			/* 
              * we do not want to report socket error, as above errors
@@ -2260,7 +2106,6 @@ static void validate_backend_connectivity(int front_end_fd)
                 FlushErrorState();
 				ereport(FATAL,
 					(errmsg("%s",error_msg),errdetail("%s",error_detail),errhint("%s",error_hint)));
-				pfree(sp);
 			}
 			PG_END_TRY();
 		}
@@ -2348,7 +2193,6 @@ retry_startup:
 	{
 		cancel_request((CancelPacket *)sp->startup_packet);
 		pool_free_startup_packet(sp);
-		connection_count_down();
 		return NULL;
 	}
 
@@ -2364,20 +2208,6 @@ retry_startup:
 		goto retry_startup;
 	}
 
-	/* GSSAPI? */
-	if (sp->major == 1234 && sp->minor == 5680)
-	{
-		ereport(DEBUG1,
-				(errmsg("selecting backend connection"),
-				 errdetail("GSSAPI request from client")));
-
-		/* sorry, Pgpool-II does not support GSSAPI yet */
-		pool_write_and_flush(frontend, "N", 1);
-
-		pool_free_startup_packet(sp);
-		goto retry_startup;
-	}
-
 	frontend->protoVersion = sp->major;
 	frontend->database = pstrdup(sp->database);
 	frontend->username = pstrdup(sp->user);
@@ -2561,161 +2391,3 @@ int pg_frontend_exists(void)
 		return -1;
 	return 0;
 }
-
-static int opt_sort(const void *a, const void *b)
-{
-	return strcmp( *(char **)a, *(char **)b);
-}
-
-/*
- * Returns PostgreSQL version.
- * The returned PgVersion struct is in static memory.
- * Caller must not modify it.
- *
- * Note:
- * Must be called while query context already exists.
- * If there's something goes wrong, this raises FATAL. So never returns to caller.
- *
- */
-PGVersion *
-Pgversion(POOL_CONNECTION_POOL * backend)
-{
-#define VERSION_BUF_SIZE	10
-	static	PGVersion	pgversion;
-	static	POOL_RELCACHE *relcache;
-	char	*result;
-	char	*p;
-	char	buf[VERSION_BUF_SIZE];
-	int		i;
-	int		major;
-	int		minor;
-
-	/*
-	 * First, check local cache. If cache is set, just return it.
-	 */
-	if (pgversion.major != 0)
-	{
-		ereport(DEBUG5,
-				(errmsg("Pgversion: local cache returned")));
-
-		return &pgversion;
-	}
-
-	if (!relcache)
-	{
-		/*
-		 * Create relcache.
-		 */
-		relcache = pool_create_relcache(pool_config->relcache_size, "SELECT version()",
-										string_register_func, string_unregister_func, false);
-		if (relcache == NULL)
-		{
-			ereport(FATAL,
-					(errmsg("Pgversion: unable to create relcache while getting PostgreSQL version.")));
-			return NULL;
-		}
-	}
-
-	/*
-	 * Search relcache.
-	 */
-	result = (char *)pool_search_relcache(relcache, backend, "version");
-	if (result == 0)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to search relcache while getting PostgreSQL version.")));
-		return NULL;
-	}
-
-	ereport(DEBUG5,
-			(errmsg("Pgversion: version string: %s", result)));
-
-	/*
-	 * Extract major version number.  We create major version as "version" *
-	 * 10.  For example, for V10, the major version number will be 100, for
-	 * V9.6 it will be 96, and so on.  For alpha or beta version, the version
-	 * string could be something like "12beta1". In this case we assume that
-	 * atoi(3) is smart enough to stop at the first character which is not a
-	 * valid digit (in our case 'b')). So "12beta1" should be converted to 12.
-	 */
-	p = strchr(result, ' ');
-	if (p == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to find the first space in the version string: %s", result)));
-		return NULL;
-	}
-
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE - 1 && p && *p != '.')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	major = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: major version: %d", major)));
-
-	/* Assuming PostgreSQL V100 is the final release:-) */
-	if (major < 6 || major > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong major version: %d", major)));
-		return NULL;
-	}
-
-	/*
-	 * If major version is 10 or above, we are done to extract major.
-	 * Otherwise extract below decimal point part.
-	 */
-	if (major >= 10)
-	{
-		major *= 10;
-	}
-	else
-	{
-		p++;
-		i = 0;
-		while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-		{
-			buf[i++] = *p++;
-		}
-		buf[i] = '\0';
-		major = major * 10 + atoi(buf);
-		ereport(DEBUG5,
-				(errmsg("Pgversion: major version: %d", major)));
-		pgversion.major = major;
-	}
-
-	/*
-	 * Extract minor version.
-	 */
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	minor = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: minor version: %d", minor)));
-
-	if (minor < 0 || minor > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong minor version: %d", minor)));
-		return NULL;
-	}
-
-
-	/*
-	 * Ok, everything looks good. Set the local cache.
-	 */
-	pgversion.major = major;
-	pgversion.minor = minor;
-	strncpy(pgversion.version_string, result, sizeof(pgversion.version_string) - 1);
-
-	return &pgversion;
-}
