@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -75,10 +75,9 @@ static int pool_get_dml_table_oid(int **oid);
 static int pool_get_dropdb_table_oids(int **oids, int dboid);
 static void pool_discard_dml_table_oid(void);
 static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool unlink, int dboid);
-
 static int pool_get_database_oid(void);
 static void pool_add_table_oid_map(POOL_CACHEKEY *cachkey, int num_table_oids, int *table_oids);
-static void pool_reset_memqcache_buffer(bool reset_dml_oids);
+static void pool_reset_memqcache_buffer(void);
 static POOL_CACHEID *pool_add_item_shmem_cache(POOL_QUERY_HASH *query_hash, char *data, int size);
 static POOL_CACHEID *pool_find_item_on_shmem_cache(POOL_QUERY_HASH *query_hash);
 static char *pool_get_item_shmem_cache(POOL_QUERY_HASH *query_hash, int *size, int *sts);
@@ -121,7 +120,7 @@ static uint32 create_hash_key(POOL_QUERY_HASH *key);
 static volatile POOL_HASH_ELEMENT *get_new_hash_element(void);
 static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element);
 static bool is_free_hash_element(void);
-static void inject_cached_message(POOL_CONNECTION *backend, char *qcache, int qcachelen);
+static void forward_pending_data(POOL_CONNECTION *frontend,POOL_CONNECTION *backend);
 
 /*
  * Connect to Memcached
@@ -197,25 +196,9 @@ void memqcache_register(char kind,
                         int data_len)
 {
 	POOL_TEMP_QUERY_CACHE *cache;
-	POOL_SESSION_CONTEXT *session_context;
-	POOL_QUERY_CONTEXT *query_context;
 
 	cache = pool_get_current_cache();
 
-	if (cache == NULL)
-	{
-		session_context = pool_get_session_context(true);
-
-		if (session_context && pool_is_query_in_progress())
-		{
-			char *query = pool_get_query_string();
-			query_context = session_context->query_context;
-
-			if (query)
-				query_context->temp_cache = pool_create_temp_query_cache(query);
-		}
-	}
-
 	pool_add_temp_query_cache(cache, kind, data, data_len);
 }
 
@@ -611,9 +594,6 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
 	int sts;
 	pool_sigset_t oldmask;
 
-	ereport(DEBUG1,
-			(errmsg("pool_fetch_from_memory_cache called")));
-
 	*foundp = false;
     
     POOL_SETMASK2(&BlockSig, &oldmask);
@@ -649,26 +629,57 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
 		POOL_CONNECTION *target_backend;
 
 		ereport(DEBUG1,
-				(errmsg("memcache: injecting cache data")));
+				(errmsg("memcache: fetching forwarding_pending_data")));
 
 		session_context = pool_get_session_context(true);
 		target_backend = CONNECTION(backend, session_context->load_balance_node_id);
-		inject_cached_message(target_backend, qcache, qcachelen);
-	}
-	else
-	{
-		/*
-		 * Send each messages to frontend
-		 */
-		send_cached_messages(frontend, qcache, qcachelen);
+		forward_pending_data(frontend, target_backend);
 	}
 
+	/*
+	 * Send each messages to frontend
+	 */
+	send_cached_messages(frontend, qcache, qcachelen);
 	pfree(qcache);
 
 	/*
-	 * Send a "READY FOR QUERY" if not in extended query.
+	 * If we are doing extended query, forward sync message from frontend to
+	 * backend. This is necessary to prevent receiving Sync message after
+	 * Sending Ready for query.
 	 */
-	if (!pool_is_doing_extended_query_message() && MAJOR(backend) == PROTO_MAJOR_V3)
+	if (pool_is_doing_extended_query_message())
+	{
+		char kind;
+		int32 len;
+		POOL_SESSION_CONTEXT *session_context;
+		POOL_CONNECTION *target_backend;
+		char buf[5];
+
+		if (pool_flush(frontend))
+			return POOL_END;
+		if (pool_read(frontend, &kind, 1))
+			return POOL_END;
+
+		ereport(DEBUG2,
+				(errmsg("memcache: fetching from memory cache: expecting sync: kind '%c'", kind)));
+		if (pool_read(frontend, &len, sizeof(len)))
+			return POOL_END;
+
+		/* Forward "Sync" message to backend */
+		session_context = pool_get_session_context(true);
+		target_backend = CONNECTION(backend, session_context->load_balance_node_id);
+		pool_write(target_backend, &kind, 1);
+		pool_write_and_flush(target_backend, &len, sizeof(len));
+
+		/* Read and discard "Ready for query" message from backend */
+		pool_read(target_backend, &kind, 1);
+		pool_read(target_backend, buf, sizeof(buf));
+	}
+
+	/*
+	 * send a "READY FOR QUERY"
+	 */
+	if (MAJOR(backend) == PROTO_MAJOR_V3)
 	{
 		signed char state;
 
@@ -678,13 +689,13 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
 		state = MASTER(backend)->tstate;
 		send_message(frontend, 'Z', 5, (char *)&state);
 	}
-
-	if (!pool_is_doing_extended_query_message() || !STREAM)
+	else
 	{
-		if (pool_flush(frontend))
-		{
-			return POOL_END;
-		}
+		pool_write(frontend, "Z", 1);
+	}
+	if (pool_flush(frontend))
+	{
+		return POOL_END;
 	}
 
 	*foundp = true;
@@ -719,10 +730,7 @@ bool pool_is_likely_select(char *query)
 		while (*query && isspace(*query))
 			query++;
 	}
-	if (! *query)
-	{
-		return false;
-	}
+	if (! *query) { return false; }
 
 	/*
 	 * Get rid of head comment.
@@ -738,10 +746,7 @@ bool pool_is_likely_select(char *query)
 			query++;
 			do_continue = true;
 		}
-		if (do_continue)
-		{
-			continue;
-		}
+		if (do_continue) { continue; }
 
 		while (*query && !strncmp(query, "\n", 2))
 		{
@@ -941,8 +946,8 @@ bool pool_is_table_in_white_list(const char *table_name)
  * Extract table oid from INSERT/UPDATE/DELETE/TRUNCATE/
  * DROP TABLE/ALTER TABLE/COPY FROM statement.
  * Returns number of oids.
- * In case of error, returns 0 (InvalidOid).
- * oids buffer (oidsp) will be discarded by subsequent call.
+ * In case of error, returns 0(InvalidOid).
+ * oids buffer(oidsp) will be discarded by subsequent call.
  */
 int pool_extract_table_oids(Node *node, int **oidsp)
 {
@@ -1030,7 +1035,7 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 		 */
 		foreach(cell, stmt->objects)
 		{
-			if (num_oids >= POOL_MAX_DML_OIDS)
+			if (num_oids > POOL_MAX_DML_OIDS)
 			{
 				ereport(LOG,
 						(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
@@ -1056,7 +1061,7 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 
 		foreach(cell, stmt->relations)
 		{
-			if (num_oids >= POOL_MAX_DML_OIDS)
+			if (num_oids > POOL_MAX_DML_OIDS)
 			{
 				ereport(LOG,
 						(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
@@ -1084,13 +1089,6 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 	oid = pool_table_name_to_oid(table);
 	if (oid > 0)
 	{
-		if (num_oids >= POOL_MAX_DML_OIDS)
-		{
-			ereport(LOG,
-					(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
-			return 0;
-		}
-
 		oids[num_oids++] = pool_table_name_to_oid(table);
 		ereport(DEBUG1,
 				(errmsg("memcache: extracting table oids: table: \"%s\" oid:%d", table, oid)));
@@ -1219,7 +1217,7 @@ static void pool_discard_dml_table_oid(void)
  * using same table oid will add to the same file. If the SELECT uses
  * multiple tables, multiple table_oid_file will be created. When
  * INSERT/UPDATE/DELETE is executed, corresponding caches must be
- * deleted (cache invalidation) (when DROP TABLE, ALTER TABLE is
+ * deleted(cache invalidation) (when DROP TABLE, ALTER TABLE is
  * executed, the caches must be deleted as well). When database is
  * dropped, all caches belonging to the database must be deleted.
  */
@@ -1294,7 +1292,7 @@ int pool_get_database_oid_from_dbname(char *dbname)
 }
 
 /*
- * Add cache id (shmem case) or hash key (memcached case) to table oid
+ * Add cache id(shmem case) or hash key(memcached case) to table oid
  * map file.  Caller must hold shmem lock before calling this function
  * to avoid file extension conflict among different pgpool child
  * process.
@@ -1495,7 +1493,7 @@ void pool_discard_oid_maps_by_db(int dboid)
 }
 
 /*
- * Read cache id (shmem case) or hash key (memcached case) from table
+ * Reading cache id(shmem case) or hash key(memcached case) from table
  * oid map file according to table_oids and discard cache entries.  If
  * unlink is true, the file will be unlinked after successful cache
  * removal.
@@ -1525,8 +1523,7 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 	/*
 	 * Create memqcache_oiddir/database_oid
 	 */
-	if (dboid == 0)
-	{
+	if (dboid == 0) {
 		dboid = pool_get_database_oid();
 		ereport(DEBUG1,
 			(errmsg("memcache invalidating query cache"),
@@ -1659,10 +1656,9 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 }
 
 /*
- * Reset SELECT data buffers.  If reset_dml_oids is true, call
- * pool_discard_dml_table_oid() to reset table oids used in DML statements.
+ * Reset SELECT data buffers
  */
-static void pool_reset_memqcache_buffer(bool reset_dml_oids)
+static void pool_reset_memqcache_buffer(void)
 {
 	POOL_SESSION_CONTEXT * session_context;
 
@@ -1686,8 +1682,7 @@ static void pool_reset_memqcache_buffer(bool reset_dml_oids)
 		 * if the query context is still under use, we cannot discard
 		 * temporary cache.
 		 */
-		if ((SL_MODE && pool_is_doing_extended_query_message()) ||
-			can_query_context_destroy(session_context->query_context))
+		if (can_query_context_destroy(session_context->query_context))
 		{
 			ereport(DEBUG1,
 				(errmsg("memcache reset buffer"),
@@ -1703,10 +1698,7 @@ static void pool_reset_memqcache_buffer(bool reset_dml_oids)
 			session_context->query_context->temp_cache = NULL;
 		}
 	}
-
-	if (reset_dml_oids)
-		pool_discard_dml_table_oid();
-
+	pool_discard_dml_table_oid();
 	pool_tmp_stats_reset_num_selects();
 }
 
@@ -1716,13 +1708,7 @@ static void pool_reset_memqcache_buffer(bool reset_dml_oids)
  */
 bool pool_is_shmem_cache(void)
 {
-	static int result = -1;
-
-	if (result == -1)
-	{
-		result = strcmp(pool_config->memqcache_method, "shmem");
-	}
-	return (result==0)?true:false;
+	return (pool_config->memqcache_method == SHMEM_CACHE)?true:false;
 }
 
 /*
@@ -2812,9 +2798,6 @@ POOL_TEMP_QUERY_CACHE *pool_create_temp_query_cache(char *query)
 
     MemoryContextSwitchTo(old_context);
 
-	ereport(DEBUG1,
-			(errmsg("pool_create_temp_query_cache: cache created: %p", p)));
-
 	return p;
 }
 
@@ -2832,10 +2815,6 @@ void pool_discard_temp_query_cache(POOL_TEMP_QUERY_CACHE *temp_cache)
 		pool_discard_buffer(temp_cache->buffer);
 	if (temp_cache->oids)
 		pool_discard_buffer(temp_cache->oids);
-
-	ereport(DEBUG1,
-			(errmsg("pool_discard_temp_query_cache: cache discarded: %p", temp_cache)));
-
 	pfree(temp_cache);
 }
 
@@ -3092,8 +3071,6 @@ static void pool_check_and_discard_cache_buffer(int num_oids, int *oids)
 			continue;
 
 		soids = (int *)pool_get_buffer(cache->oids, &len);
-		if (!soids || !len)
-			continue;
 
 		for(j=0;j<cache->num_oids;j++)
 		{
@@ -3116,9 +3093,8 @@ static void pool_check_and_discard_cache_buffer(int num_oids, int *oids)
 }
 
 /*
- * At Ready for Query or Comand Complete handle query cache.  For streaming
- * replication mode and extended query at Comand Complete handle query cache.
- * For other case At Ready for Query handle query cache.
+ * At Ready for Query handle query cache.
+ * Supposed to be called from ReadyForQuery().
  */
 void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *node, char state)
 {
@@ -3152,7 +3128,6 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			 */
 			if (!pool_is_cache_exceeded())
 			{
-				POOL_TEMP_QUERY_CACHE *cache;
 				/*
 				 * If we are not inside a transaction, we can
 				 * immediately register to cache storage.
@@ -3164,33 +3139,30 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 				cache_buffer =  pool_get_current_cache_buffer(&len);
 				if (cache_buffer)
 				{
-					if (session_context->query_context->skip_cache_commit == false)
+					if (pool_commit_cache(backend, query, cache_buffer, len, num_oids, oids) != 0)
 					{
-						if (pool_commit_cache(backend, query, cache_buffer, len, num_oids, oids) != 0)
-						{
-							ereport(WARNING,
-									(errmsg("ReadyForQuery: pool_commit_cache failed")));
-						}
-					}
-					/*
-					 * Reset temporary query cache buffer. This is
-					 * necessary if extended query protocol is used and a
-					 * bind/execute message arrives which uses a statement
-					 * created by prior parse message. In this case since
-					 * the temp_cache is not initialized by a parse
-					 * message, messages are added to pre existing temp
-					 * cache buffer. The problem was found in bug#152.
-					 * http://www.pgpool.net/mantisbt/view.php?id=152
-					 */
-					cache = pool_get_current_cache();
-					ereport(DEBUG1,
-							(errmsg("pool_handle_query_cache: temp_cache: %p", cache)));
-					pool_discard_temp_query_cache(cache);
+						ereport(WARNING,
+								(errmsg("ReadyForQuery: pool_commit_cache failed")));
 
-					if (STREAM && pool_is_doing_extended_query_message())
-						session_context->query_context->temp_cache = NULL;
+					}
 					else
+					{
+						/*
+						 * Reset temporary query cache buffer. This is
+						 * necessary if extended query protocol is used and a
+						 * bind/execute message arrives which uses a statement
+						 * created by prior parse message. In this case since
+						 * the temp_cache is not initialized by a parse
+						 * message, messages are added to pre existing temp
+						 * cache buffer. The problem was found in bug#152.
+						 * http://www.pgpool.net/mantisbt/view.php?id=152
+						 */
+						POOL_TEMP_QUERY_CACHE *cache;
+						cache = pool_get_current_cache();
+						pool_discard_temp_query_cache(cache);
 						session_context->query_context->temp_cache = pool_create_temp_query_cache(query);
+					}
+
 					pfree(cache_buffer);
 				}
 				pool_shmem_unlock();
@@ -3201,7 +3173,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			pool_stats_count_up_num_selects(1);
 
 			/* Reset temp buffer */
-			pool_reset_memqcache_buffer(true);
+			pool_reset_memqcache_buffer();
 		}
 		else
 		{
@@ -3253,7 +3225,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 	else if (is_rollback_query(node))	/* Rollback? */
 	{
 		/* Discard buffered data */
-		pool_reset_memqcache_buffer(true);
+		pool_reset_memqcache_buffer();
 	}
 	else if (is_commit_query(node))		/* Commit? */
 	{
@@ -3305,7 +3277,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 		/* Count up number of SELECT stats */
 		pool_stats_count_up_num_selects(pool_tmp_stats_get_num_selects());
 
-		pool_reset_memqcache_buffer(true);
+		pool_reset_memqcache_buffer();
 	}
 	else		/* Non cache safe queries */
 	{
@@ -3316,7 +3288,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			{
 				/* Count up SELECT stats */
 				pool_stats_count_up_num_selects(1);
-				pool_reset_memqcache_buffer(true);
+				pool_reset_memqcache_buffer();
 			}
 			else
 			{
@@ -3339,7 +3311,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 				pool_invalidate_query_cache(num_oids, oids, true, dboid);
 				pool_discard_oid_maps_by_db(dboid);
 				pool_shmem_unlock();
-				pool_reset_memqcache_buffer(true);
+				pool_reset_memqcache_buffer();
 
 				pfree(oids);
 				ereport(DEBUG2,
@@ -3368,7 +3340,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 					pool_invalidate_query_cache(num_oids, oids, true, 0);
 					pool_shmem_unlock();
 					POOL_SETMASK(&oldmask);
-					pool_reset_memqcache_buffer(true);
+					pool_reset_memqcache_buffer();
 				}
 				else
 				{
@@ -3379,16 +3351,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 					 * if DML/DDL modifies the TABLE which SELECT uses.
 					 */
 					pool_check_and_discard_cache_buffer(num_oids, oids);
-					pool_reset_memqcache_buffer(false);
-				}
-			}
-			else if (num_oids == 0)
-			{
-				/* 
-				 * It is also necessary to clear cache buffers in case of
-				 * no oid queries (like BEGIN, CHECKPOINT, VACUUM, etc) too.
-				 */
-				pool_reset_memqcache_buffer(true);
+				} 
 			}
 		}
 	}
@@ -3923,7 +3886,7 @@ POOL_SHMEM_STATS *pool_get_shmem_storage_stats(void)
 	mystats.cache_stats.num_selects = stats->num_selects;
 	mystats.cache_stats.num_cache_hits = stats->num_cache_hits;
 
-	if (strcmp(pool_config-> memqcache_method, "shmem"))
+	if (pool_config->memqcache_method == SHMEM_CACHE)
 		return &mystats;
 
 	/* number of total hash entries */
@@ -3986,38 +3949,23 @@ POOL_SHMEM_STATS *pool_get_shmem_storage_stats(void)
 }
 
 /*
- * Inject cached message to the target backend buffer to pretend as if backend
- * actually replies with Data row and Command Complete message.
+ * Send flash message to backend and forward any response to frontend.
  */
-static void inject_cached_message(POOL_CONNECTION *backend, char *qcache, int qcachelen)
+void forward_pending_data(POOL_CONNECTION *frontend, POOL_CONNECTION *backend)
 {
 	char kind;
 	int len;
 	char *buf;
-	int timeout;
-	int i = 0;
-	bool is_prepared_stmt = false;
-	POOL_SESSION_CONTEXT *session_context;
-	POOL_QUERY_CONTEXT* query_context;
-	POOL_PENDING_MESSAGE *msg;
 
-	session_context = pool_get_session_context(false);
-	query_context = session_context->query_context;
-	msg = pool_pending_message_find_lastest_by_query_context(query_context);
-
-	if (msg)
+	/* If are doing extended query and we are in streaming replication
+	 * mode, we need to retrieve any response from backend.
+	 */
+	if (!pool_is_pending_response())
 	{
-		/*
-		 * If pending message found, we should extract target backend from it
-		 */
-		int backend_id;
-
-		backend_id = pool_pending_message_get_target_backend_id(msg);
-		backend = CONNECTION(session_context->backend, backend_id);
-		timeout = -1;
+		ereport(DEBUG1,
+				(errmsg("forward_pending_data: no peding data")));
+		return;
 	}
-	else
-		timeout = 0;
 
 	/* Send flush messsage to backend to retrieve response of backend */
 	pool_write(backend, "H", 1);		
@@ -4025,89 +3973,44 @@ static void inject_cached_message(POOL_CONNECTION *backend, char *qcache, int qc
 	pool_write_and_flush(backend, &len, sizeof(len));
 
 	/*
-	 * Push any response from backend
+	 * Then forward any response from backend
 	 */
 	for(;;)
 	{
 		pool_read(backend, &kind, 1);
 		ereport(DEBUG1,
-				(errmsg("inject_cached_message: push message kind: '%c'", kind)));
-		if (msg &&
-			((kind == 'T' && msg->type == POOL_DESCRIBE) ||
-			 (kind == '2' && msg->type == POOL_BIND)))
-		{
-			/* Pending message seen. Now it is likely to end of pending data */
-			timeout = 0;
-		}
-		pool_push(backend, &kind, sizeof(kind));
+				(errmsg("forward_pending_data: forwarding kind: '%c'", kind)));
+		pool_write(frontend, &kind, 1);
+
 		pool_read(backend, &len, sizeof(len));
-		pool_push(backend, &len, sizeof(len));
-		if ((ntohl(len)-sizeof(len)) > 0)
+		ereport(DEBUG1,
+				(errmsg("forward_pending_data: forwarding len: '%d'", ntohl(len))));
+		pool_write(frontend, &len, sizeof(len));
+
+		len = ntohl(len);
+		if ((len - sizeof(len)) > 0)
 		{
-			buf = pool_read2(backend, ntohl(len)-sizeof(len));
-			pool_push(backend, buf, ntohl(len)-sizeof(len));
+			len -= sizeof(len);
+			ereport(DEBUG1,
+					(errmsg("forward_pending_data: fowarding rest of packet. len:%d", len)));
+			buf = palloc(len);
+			pool_read(backend, buf, len);
+			pool_write(frontend, buf, len);
 		}
 
 		/* check if there's any pending data */
 		if (!pool_ssl_pending(backend) && pool_read_buffer_is_empty(backend))
 		{
-			pool_set_timeout(timeout);
+			pool_set_timeout(0);
 			if (pool_check_fd(backend) != 0)
 			{
 				ereport(DEBUG1,
-						(errmsg("inject_cached_message: select shows no pending data")));
+						(errmsg("forward_pending_data: select shows no pending data")));
 				pool_set_timeout(-1);
 				break;
 			}
 			pool_set_timeout(-1);
 		}
 	}
-
-	/*
-	 * Inject row data and command complete
-	 */
-	while (i < qcachelen)
-	{
-		char tmpkind;
-		int tmplen;
-		char *p;
-
-		tmpkind = qcache[i];
-		i++;
-
-		memcpy(&tmplen, qcache+i, sizeof(tmplen));
-		i += sizeof(tmplen);
-		len = ntohl(tmplen);
-		p = qcache + i;
-		i += len - sizeof(tmplen);
-
-		/* No need to cache PARSE and BIND responses */
-		if (tmpkind == '1' || tmpkind == '2')
-		{
-			is_prepared_stmt = true;
-			continue;
-		}
-
-		/*
-		 * In the prepared statement execution, there is no need to send
-		 * 'T' response to the frontend.
-		 */
-		if (is_prepared_stmt && tmpkind == 'T')
-		{
-			continue;
-		}
-
-		/* push message */
-		ereport(DEBUG1,
-			(errmsg("inject_cached_message: push cached messages: '%c' len: %d", tmpkind, len)));
-		pool_push(backend, &tmpkind, 1);
-		pool_push(backend, &tmplen, sizeof(tmplen));
-		if (len > 0)
-			pool_push(backend, p, len - sizeof(tmplen));
-	}
-
-	/*
-	 * Pop data.
-	 */
-	pool_pop(backend, &len);
+	pool_flush(frontend);
 }
